<?php
/**
 * iMaQma Helpdesk Component
 * www.imaqma.com
 *
 * @package iMaQma Helpdesk
 * @copyright (C) 2000-2011 Components Lab, Lda.
 * @license GNU/GPL
 * 
 * $Id: admin.glossary.php 337 2012-01-22 17:24:30Z pdaniel $
 * $LastChangedDate: 2012-01-22 17:24:30 +0000 (dom, 22 Jan 2012) $
 *
 */

/** ensure this file is being included by a parent file */
defined( '_JEXEC' ) or die( 'Direct Access to this location is not allowed.' );

require_once( "components/com_maqmahelpdesk/html/admin.$function.html.php" );

// get parameters from the URL or submitted form
$cid = JRequest::getVar( 'cid', array(0), '', 'array' );
if (!is_array( $cid )) {
	$cid = array(0);
}

// Activities logger
mqm_activitylog('admin', 'glossary', $task, $cid[0]);

switch ($task) {
	case "new":
		editGlossary( 0 );
		break;

	case "edit":
		editGlossary( $cid[0] );
		break;

	case "save":
		saveGlossary();
		break;

	case "remove":
		removeGlossary( $cid );
		break;

	case "publish":
		publishGlossary( $cid, 1 );
		break;

	case "unpublish":
		publishGlossary( $cid, 0 );
		break;

	default:
		showGlossary();
		break;
}

/**
* Compiles a list of priorities
* @param string The component name
*/
function showGlossary() {
	$database = JFactory::getDBO();
	$mainframe = JFactory::getApplication();
	$limit = $mainframe->getUserStateFromRequest( "global.list.limit", 'limit', $mainframe->getCfg('list_limit'), 'int');
	$limitstart = intval( JRequest::getVar( 'limitstart', 0, '', 'int' ) );

	// get the total number of records
	$database->setQuery( "SELECT count(*) FROM #__support_glossary" );
	$total = $database->loadResult();
	echo $database->getErrorMsg();

	jimport('joomla.html.pagination');;
	$pageNav = new JPagination( $total, $limitstart, $limit );

	$database->setQuery( "SELECT id, term, description, published, anonymous_access"
	. "\nFROM #__support_glossary"
	. "\nORDER BY term",
	$pageNav->limitstart, $pageNav->limit
	);

	$rows = $database->loadObjectList();
	if ($database->getErrorNum()) {
		echo $database->stderr();
		return false;
	}

	glossary_html::show( $rows, $pageNav );
}

/**
* Compiles information to add or edit a priority
* @param database A database connector object
* @param string The component name
*/
function editGlossary( $uid=0 ) {
	$database = JFactory::getDBO();
	$row = new mosSupportGlossary( $database );
	// load the row from the db table
	$row->load( $uid );

	$captions = array( JText::_('MQ_NO'), JText::_('MQ_YES') );
	$values = array( '0', '1' );
	
	$lists['published'] = mqm_switchcheckbox('radio', 'published', $captions, $values, $row->published, 'switch');
	$lists['anonymous'] = mqm_switchcheckbox('radio', 'anonymous_access', $captions, $values, $row->anonymous_access, 'switch');
	
	glossary_html::edit( $row, $lists );
}

/**
* Saves the workgroup
* @param database A database connector object
* @param string The name of the component
*/
function saveGlossary() {
	$database = JFactory::getDBO();
	$mainframe = JFactory::getApplication();
	$row = new mosSupportGlossary( $database );

	if (!$row->bind( $_POST )) {
		echo "<script type='text/javascript'> alert('".$row->getErrorMsg()."'); window.history.go(-1); </script>\n";
		exit();
	}
	if (!$row->check()) {
		echo "<script type='text/javascript'> alert('".$row->getErrorMsg()."'); window.history.go(-1); </script>\n";
		exit();
	}
	if (!$row->store()) {
		echo "<script type='text/javascript'> alert('".$row->getErrorMsg()."'); window.history.go(-1); </script>\n";
		exit();
	}

	$mainframe->redirect( "index.php?option=com_maqmahelpdesk&task=glossary" );
}

/**
* Deletes one or more categories from the categories table
* @param database A database connector object
* @param string The name of the priority section
* @param array An array of unique priority id numbers
*/
function removeGlossary( $cid ) {
	$database = JFactory::getDBO();
	$mainframe = JFactory::getApplication();

	if (count( $cid ) < 1) {
		echo "<script type='text/javascript'> alert('".JText::_('glossary_delete')."'); window.history.go(-1);</script>\n";
		exit;
	}

	if (count( $cid )) {
		$cids = implode( ',', $cid );
		$database->setQuery( "DELETE FROM #__support_glossary WHERE id IN (".$database->quote($cids).")" );
		if (!$database->query()) {
			echo "<script type='text/javascript'> alert('".$database->getErrorMsg()."'); window.history.go(-1); </script>\n";
		}
	}

	$mainframe->redirect( "index.php?option=com_maqmahelpdesk&task=glossary" );
}

/**
* Publishes or Unpublishes one or more workgroups
* @param database A database connector object
* @param array An array of unique workgroup id numbers
* @param integer 0 if unpublishing, 1 if publishing
* @param string The name of the component
*/
function publishGlossary( $cid=null, $publish=1 ) {
	$database = JFactory::getDBO();
	$mainframe = JFactory::getApplication();

	if ( !is_array( $cid ) || count( $cid ) < 1 ) {
		$action = $publish ? 'publish' : 'unpublish';
		echo "<script type='text/javascript'> alert('".JText::_('glossary_action')." ".$action."'); window.history.go(-1);</script>\n";
		exit;
	}

	$cids = implode( ',', $cid );
	$count = count( $cid );

	$database->setQuery( "UPDATE #__support_glossary SET published='$publish' WHERE id IN (".$database->quote($cids).")" );

	if (!$database->query()) {
		echo "<script type='text/javascript'> alert('".$database->getErrorMsg()."'); window.history.go(-1); </script>\n";
		exit();
	}

	$mainframe->redirect( "index.php?option=com_maqmahelpdesk&task=glossary" );
}
?>