<?php
/**
 * @copyright	Copyright (C) 2009 - 2009 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
 * @license		GNU/GPL, see LICENSE.php
 * @package		PayPlans
 * @subpackage	acajoom
 * @contact 	payplans@readybytes.in
 * 
 */
if(defined('_JEXEC')===false) die();

class PayplansAppAcajoom extends PayplansApp
{
	protected $_location	= __FILE__;
	protected $_resource	= 'com_acajoom.list';

	public function _isApplicable(PayplansIfaceApptriggerable $refObject)
	{
		return JFolder::exists(JPATH_SITE .DS.'components'.DS.'com_acajoom');
	}
	
	public function onPayplansSubscriptionAfterSave($prev, $new)
	{
		// no need to trigger if previous and current state is same
		if($prev != null && $prev->getStatus() == $new->getStatus()){
			return true;
		}

		$userid			= $new->getBuyer();
		$subid  		= $new->getId();
		$removeFromList = array();
		
		//on subscription status active
		if( $new->getStatus()   == PayplansStatus::SUBSCRIPTION_ACTIVE){
			$addToList 	    	= $this->getAppParam('addToListonActive');
			$removeFromList[]   = $this->getAppParam('addToListonHold');
			$removeFromList[]	= $this->getAppParam('addToListonExpire');
			return $this->_addOrRemoveFromList($subid, $userid, $addToList, $removeFromList);
		}

		//on subscription status hold
		if( $new->getStatus()   == PayplansStatus::SUBSCRIPTION_HOLD){
			$addToList 			= $this->getAppParam('addToListonHold');
			$removeFromList[]   = $this->getAppParam('addToListonActive');
			$removeFromList[]	= $this->getAppParam('addToListonExpire');
			return $this->_addOrRemoveFromList($subid, $userid, $addToList, $removeFromList);
		}
		
		//on subscription status expire
		if( $new->getStatus()   == PayplansStatus::SUBSCRIPTION_EXPIRED){
			$addToList 			= $this->getAppParam('addToListonExpire');
			$removeFromList[]   = $this->getAppParam('addToListonActive');
			$removeFromList[]	= $this->getAppParam('addToListonHold');
			return $this->_addOrRemoveFromList($subid, $userid, $addToList, $removeFromList);
		}

	}
	
	/**
	 * add or remove from the particular list
	 */
	function _addOrRemoveFromList($subid, $userid, $addToList, $removeFromList)
	{
		$subscriber = $this->getSubscriberID($userid);
		
		if (!$subscriber) {
			$this->createSubscriber( $userid );
			$subscriber = $this->getSubscriberID( $userid );
		}

		if ( !$subscriber ) {
			return null;
		}

		$this->deleteFromList( $subid, $subscriber, $removeFromList, $userid);
		$this->addToList( $subid, $subscriber, $addToList, $userid);
	}

	/**
	 * Add subscriber if not exist
	 */
	function createSubscriber( $userid )
	{
		$user  = XiFactory::getUser($userid);
		$query = new XiQuery();
		$query->insert ("`#__acajoom_subscribers`")
			  ->set("user_id   = '". $userid ."'")
			  ->set("name      = '". $user->name."'")
			  ->set("email	   = '". $user->email."'")
			  ->set("confirmed = 1")
			  ->set("subscribe_date = '". date( 'Y-m-d H:i:s' )."'")
			  ->dbLoadQuery()
			  ->query();
	}

	/**
	 *  Get subscriber ID corresponding the given userid
	 */
	function getSubscriberID( $userid )
	{
		$query = new XiQuery();
		return $query->select("`id`")
					 ->from("`#__acajoom_subscribers`")
					 ->where("`user_id` =".$userid)
					 ->dbLoadQuery()
					 ->loadResult();
	}

	/**
	 * Add subscriber to list
	 */
	function addToList($subid, $subscriber_id, $list_ids, $userid )
	{
		if(empty($list_ids)){
			return true;
		}
		
		$list_ids = is_array($list_ids) ? $list_ids : array($list_ids); 
				
		$values = array();	
		foreach($list_ids as $id){
			$subscribers = $this->_getSubscribersList($id);
			
			//if subscriber already exists then addToResource only 
			if(in_array($subscriber_id, $subscribers)){
				$this->_addToResource($subid, $userid, $id, $this->_resource);
				continue;
			}

			$listRecord = $this->_getListType($id);
			$query = new XiQuery();
			$query->insert("`#__acajoom_queue`")
			      ->set(" type          = ". $listRecord['list_type'])
			      ->set(" subscriber_id = ". $subscriber_id)
			      ->set(" list_id       = ". $id)
			      ->set(" acc_level     = ". $listRecord['acc_id'])
			      ->dbLoadQuery()
			      ->query();
			$this->_addToResource($subid, $userid, $id, $this->_resource);
		}

		return true;
	}

	/**
	 * get list type and access level
	 */
	protected function _getListType($listId)
	{
		$query = new XiQuery();
		return $query->select("`list_type`,`acc_id`")
					 ->from("`#__acajoom_lists`")
					 ->where("`id` =".$listId)
					 ->dbLoadQuery()
					 ->loadAssoc();		
	}
	
	/**
	 *  Delete subscriber from list
	 */
	function deleteFromList($subid, $subscriber_id, $list_ids, $userid )
	{
		if(empty($list_ids)){
			return true;
		}
		
		$list_ids = is_array($list_ids) ? $list_ids : array($list_ids); 

		foreach ($list_ids as $list_id){
			if(is_array($list_id)){
				$this->deleteFromList($subid, $subscriber_id, $list_id, $userid);
			}
			else {
				if($this->_removeFromResource($subid, $userid, $list_id, $this->_resource)){
					$query = new XiQuery();
					$query->delete()
						  ->from('`#__acajoom_queue`')
						  ->where('`subscriber_id` = ' .$subscriber_id .' ', 'AND')
						  ->where('`list_id` = ' .$list_id)
						  ->dbLoadQuery()
						  ->query();
				}
			}
		}
		
		return true;
	}
	
	/**
	 * Get all subscribers in the given list
	 */
	protected function _getSubscribersList($listid)
	{
		$query = new XiQuery();
		return $query->select("`subscriber_id`")
		      		 ->from("`#__acajoom_queue`")
		      		 ->where("`list_id` = ".$listid)
		      		 ->dbLoadQuery()
		     		 ->loadResultArray();
	}
	
	/**
	 * get list name
	 */
	function getNameFromResourceValue($resource, $value)
	{
		// if its a different resource
		if($resource != $this->_resource){
			return false;
		}
		
		$query = new XiQuery();
		return $query->select("`list_name`")	
		      		 ->from("`#__acajoom_lists`")	
		     		 ->where("`id` = ".$value)
		     		 ->dbLoadQuery()
		      		 ->loadResult();
	}

	/**
	 * render widget html
	 */
	public function renderWidgetHtml()
	{   
		$userid     = XiFactory::getUser()->id;
		//get user's acajoom lists
		$subcriberid   = $this->getSubscriberID($userid);
		
		if(!$subcriberid){
			return '';
		}
		
	    $acajoom_lists = $this->_getUserAcajoomList($subcriberid);
	 
	  	if(empty($acajoom_lists)){
	    	return '';
	    }
	    $this->assign('acajoom_lists',$acajoom_lists);
        return $this->_render('widgethtml');
	}	
	
	/**
	 * get list in which given subscriber exists
	 */
	protected function _getUserAcajoomList($subscriberid)
	{
		$db = XiFactory::getDBO();
		$user_lists = array();
		
		//Add Acajoom List
		$query      = " SELECT `list_name`,`list_id`,`list_type`"
			        . " FROM #__acajoom_queue as queue, #__acajoom_lists as list"
			        . " WHERE queue.`subscriber_id` = " . $subscriberid . " AND queue.`list_id` = list.`id`";
			       
        $db->setQuery( $query );
        return $db->loadObjectList();
	}
	
}