<?php
/**
* @copyright	Copyright (C) 2009 - 2009 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
* @license		GNU/GPL, see LICENSE.php
* @package		PayPlans
* @subpackage	Frontend
* @contact 		payplans@readybytes.in
* 
* 
*/
if(defined('_JEXEC')===false) die();

class PayplansAppK2item extends PayplansApp
{
	protected $_location = __FILE__;
	const DO_NOTHING     = -1;
	const ALLOWED 	     =  1;
	const BLOCKED        =  0;
	
	public function isApplicable( $refObject, $eventName='')
	{
		// applicable only if k2 component exists
		if(!JFolder::exists(JPATH_SITE .DS.'components'.DS.'com_k2')){
			return false;
		}

		$userId = XiFactory::getUser()->id;
		// when no user is logged-in
		if($userId == 0){
			return true;
		}
		$user = PayplansUser::getInstance($userId);
		
		//if applicable to all is false then check plan v/s apps
		if($this->getParam('applyAll',false) == false){
		 	// if object is of interest as per plans selected
			$ret = array_intersect($this->getPlans(), $user->getPlans());
			if(count($ret) > 0 ){
				return false;
			}
		}
		return true;

	}
	
	function onPayplansAccessCheck(PayplansUser $user)
	{
		//return if user is admin
		if($user->isAdmin())
		{
			return false;
		}
		
		$option = JRequest::getVar('option');
		
		if($option != 'com_k2'){
			return false;
		}
		
	
		$view 		= JRequest::getVar('view');
		$layout 	= JRequest::getVar('layout');
		$task 		= JRequest::getVar('task');
		$cat_id		= strtok(JRequest::getVar('id'),':');
		$Itemid		= JRequest::getVar('Itemid');
		
		if($view == 'itemlist' && $task == 'category'){
			
//			if(empty($layout) || empty($cat_id)){
//				$msg = XiText::_('COM_PAYPLANS_APP_K2_YOU_ARE_NOT_ALLOWED_TO_VIEW_CATEGORY');
//				PayplansFactory::getApplication()->redirect(XiRoute::_('index.php?option=com_payplans&view=dashboard'), $msg);
//			}
			
			//it should also restricted for parent categories also
			$parentCat = PayplansAppK2category::_getParentCategories($cat_id);
			$parentCat[] = $cat_id;
			
			$apprestriction = $this->getAppParam('restricted_category');
			if(empty($apprestriction))
			{
				return true;
			}
			$apprestriction = (is_array($apprestriction))? $apprestriction : array($apprestriction);
			if(array_intersect($parentCat, $apprestriction)){
			
				//if app restricted for specific plans then need to check user plans
				if($this->getParam('applyAll',false) == false){
				 	// if object is of interest as per plans selected
					$ret = array_intersect($this->getPlans(), $user->getPlans());
					if(count($ret) > 0 ){
						return true;
					}
					
					$msg = XiText::_('COM_PAYPLANS_APP_K2_YOU_ARE_NOT_ALLOWED_TO_VIEW_CATEGORY');
					PayplansFactory::getApplication()->redirect(XiRoute::_('index.php?option=com_payplans&view=dashboard'), $msg);
				}			
				
				//if appy all is true then user can have any plan
				if(count($user->getPlans()) == 0){
					$msg = XiText::_('COM_PAYPLANS_APP_K2_YOU_ARE_NOT_ALLOWED_TO_VIEW_CATEGORY');
					PayplansFactory::getApplication()->redirect(XiRoute::_('index.php?option=com_payplans&view=dashboard'), $msg);
				}
			}
		}
		return true;
		
	} 


	function onK2PrepareContent($item, $params, $limitstart)
	{
		if(!is_a($item,'stdClass')){
			return true;
		}

		$itemId       = array($item->id);
	    $allowedOrNot = plgPayplansK2::isUserAllowed($itemId,'k2item','k2item');
        if(in_array(self::ALLOWED, $allowedOrNot)){
       		return true;
        }

        if(in_array(self::BLOCKED, $allowedOrNot)){
			$url             = XiRoute::_('index.php?option=com_payplans&view=plan');
		    $item->text      = $item->introtext. '<a href="'.$url.'">'.XiText::_('COM_PAYPLANS_APP_K2ITEM_SUBSCRIBE_PLAN').'</a>';
			$item->introtext = '';
			$item->fulltext  = '';
			$item->link      = $url;
			return true;
       }
	}
	
	 //render Widget
	public function renderWidgetHtml()
	{   
		$userid         = XiFactory::getUser()->id;
		if(!$userid){
			return '';
		}

		$k2Items   = array();

		//get k2 categories
		if($userid){
	       $k2Items = $this->_getAllowedK2Items($userid);
		}

        if(empty($k2Items)){
        	return '';
        }
	    $this->assign('k2_items',$k2Items);
        $data = $this->_render('widgethtml');
        return $data;
	}	
	
	function _getAllowedK2Items($userid)
	{   
		
		$query      =  new XiQuery();
		$items      =  $query->select(' `id`, `title` ')
		               ->from('`#__k2_items`')
		               ->where('`published` = 1')
		               ->dbLoadQuery()
		               ->loadObjectList('id');

	  	//check which item is allowed to the user
        foreach($items as $item){
	        $allowedOrNot  = plgPayplansK2::isUserAllowed(array($item->id),'k2item','k2item');

			// if some one allows then its allowed
			if(in_array(self::ALLOWED, $allowedOrNot)) {
				 continue;
			}

			// if no one allows but some one blocks then its blocked
			if(in_array(self::BLOCKED, $allowedOrNot)) {
	     	   	 unset($items[$item->id]);
    	    }
    	     
			//else nothing to do
        }

        return $items;
	 }
	 
 	public static function getK2Items()
	{
		$query = new XiQuery();
		return $query->select(' `id` as item_id, `title` ')
		             ->from('`#__k2_items`')
		             ->where('`trash` = 0')
		             ->dbLoadQuery()
		             ->loadObjectList('item_id');
	}
}

class PayplansAppK2itemFormatter extends PayplansAppFormatter
{
	//get Rules 
	function getVarFormatter()
	{
		$rules = array('_appplans'       => array('formatter'=> 'PayplansAppFormatter',
										       'function' => 'getAppPlans'),
					   'app_params'      => array('formatter'=> 'PayplansAppK2itemFormatter',
										       'function' => 'getFormattedParams'));
		return $rules;
	}
	
	// formatt params
	function getFormattedParams($key,$value,$data)
	{
		//do nothing if component is not installed
		if(!JFolder::exists(JPATH_SITE .DS.'components'.DS.'com_k2')){
			return false;
		}
		$items  = PayplansAppK2item::getK2Items();
		$params  	= PayplansHelperParam::iniToArray($value);
		if(empty($params['k2item']))
		{
			return false;
		}
		// categories
		if(is_array($params['k2item'])){
		foreach($params['k2item'] as $param){
			$item[] = $items[$param]->title;
		}
		$params['k2item']    = $item;
		}
		else{
			$params['k2item'] = $items[$params['k2item']]->title;
		}
		$value   = PayplansHelperParam::arrayToIni($params);
		
	}
}
